/*!
 * \file main.c
 * \brief Mdulo Timer/PWM - PWM.
 * \mainpage Projeto de varia&ccedil;&atilde;o de cores de um Led RGB.
 * \details Este c&oacute;digo ilustra a configura&ccedil;&atilde;o dos canais do mdulo TPM para 
 *          variar a intensidade dos leds RGB (PWM) de forma a ter uma maior gama de cores. 
 * 			(Leitura recomendada: Se&ccedil;&otilde;es 31.4.6 e 31.4.7 de KL25 Sub-Family Reference Manual
 * 			 Se&ccedil;&atilde;o 12.3 de Kinetis L Peripheral Module Quick Reference)
 * \author Wu Shin-Ting
 * \date 26/09/2016
 * \note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
 */

#include <stdlib.h>
#include "port.h"
#include "ledRGB.h"
#include "tpm.h"
#include "atrasos.h"

#define COUNTER_OVF 655         ///< valor m&aacute;ximo do TPMx_CNT

int main(void)
{	
	/*!
	 * Cores adaptadas a partir de https://pymolwiki.org/index.php/Color_Values
	 */
    cor_pf cores[12] = {
    		{1.0, 0.0, 0.0},       ///< vermelho
    		{1.0, 0.42, 0.42},     ///< salmao 
    		{1.0, 1.0, 0.15},      ///< amarelo claro
    	    {0.0, 1.0, 0.0},       ///< verde
    	    {0.05, 0.9, 0.05},     ///< verde floresta 
    	    {0., 0.75, 0.75},      ///< azul berilo    
    	    {0.02, 0.3, 1.0},      ///< azul celeste 
    	    {0.0, 0.0, 1.0},       ///< azul           
    		{0.6, 0.1, 0.6},       ///< violeta        
    	    {0.65, 0.37, 0.27},    ///< marrom         
    	    {0.698, 0.35, 0.40},   ///< tijolo         
    		{1.0, 1.0, 1.0}        ///< branco
    };
    uint8_t i,j;
    
    /*! 
     * Inicializa os tr&ecirc;s pinos dos leds para serem controlados por TPM
     */
	initPort(B);                          ///< leds R e G
	initPort(D);                          ///< led B
	setaMux(ePORTB_PCR18, 0x3);           ///< TPM2_C0
	setaMux(ePORTB_PCR19, 0x3);			  ///< TPM2_C1
	setaMux(ePORTD_PCR1, 0x4);			  ///< TPM0_C1
	
	/*!
	 * Configura TPM: 
	 * Pr&eacute;-escala=0b101;
	 * Contagem M&aacute;xima=COUNTER_OVF
	 * Contagem crescente
	 */
	initTPM(0,(uint8_t)0b101,(uint16_t)(COUNTER_OVF),0);  
	initTPM(2,(uint8_t)0b101,(uint16_t)(COUNTER_OVF),0);
	
	/*! Inicializa os canais desabilitados */ 
	initTPM0Cn(1,0b0000,(unsigned short)(1.1*COUNTER_OVF));                                /*! led azul */
    initTPM2Cn(0,0b0000,(unsigned short)(1.1*COUNTER_OVF));                                /*! led vermelho */
    initTPM2Cn(1,0b0000,(unsigned short)(1.1*COUNTER_OVF));                                /*! led verde */
    
	for(;;) {
		for (i=0; i<12; i++) {
			/*!  Recarga dos contadores dos canais PWM */
			
			TPM2_SC |= 	TPM_SC_CMOD(0x0);           ///< desabilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
			TPM0_SC |= 	TPM_SC_CMOD(0x0);           ///< desabilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
			TPM2_C0V = TPM_CnV_VAL((unsigned short)(cores[i].vermelho*COUNTER_OVF));     ///< % R
			TPM2_C1V = TPM_CnV_VAL((unsigned short)(cores[i].verde*COUNTER_OVF));        ///< % G
			TPM0_C1V = TPM_CnV_VAL((unsigned short)(cores[i].azul*COUNTER_OVF));         ///< % B
			TPM2_SC |= 	TPM_SC_CMOD(0x1);           ///< habilita clock LPTPM em contagem crescente 
			TPM0_SC |= 	TPM_SC_CMOD(0x1);           ///< habilita clock LPTPM em contagem crescente 
		
			for (j=0; j<2;j++) {
				/*! Sele&ccedil;&atilde;o de modo, transi&ccedil;&atilde;o e n&iacute;vel (Modo: PWM alinhado a borda) */ 
				/*! Led R */
				TPM2_C0SC |= (TPM_CnSC_MSB_MASK |         	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |          			///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );         			///< ELSA=TPM2_C1SC[2]=1
				TPM2_C0SC &= ~TPM_CnSC_MSA_MASK;          	///< MSA=TPM2_C1SC[4]=0
				/*! Led G */
				TPM2_C1SC |= (TPM_CnSC_MSB_MASK |       	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |        			///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );       			///< ELSA=TPM2_C1SC[2]=1
				TPM2_C1SC &= ~TPM_CnSC_MSA_MASK;        	///< MSA=TPM2_C1SC[4]=0
				/*! Led B */
				TPM0_C1SC |= (TPM_CnSC_MSB_MASK |         	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |        			///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );       			///< ELSA=TPM2_C1SC[2]=1
				TPM0_C1SC &= ~TPM_CnSC_MSA_MASK;         	///< MSA=TPM2_C1SC[4]=0

				delay10us(100000);
		
				/*! Sele&ccedil;&atilde;o de modo, transi&ccedil;&atilde;o e n&iacute;vel (desabilita os 3 canais) */ 
				TPM2_C0SC &= ~(TPM_CnSC_MSB_MASK |      ///< MSB=TPM2_C0SC[5]=0
					TPM_CnSC_MSA_MASK |         		///< MSA=TPM2_C0SC[4]=0
					TPM_CnSC_ELSB_MASK |        		///< ELSB=TPM2_C0SC[3]=0
					TPM_CnSC_ELSA_MASK );       		///< ELSA=TPM2_C0SC[2]=0
				TPM2_C1SC &= ~(TPM_CnSC_MSB_MASK |      ///< MSB=TPM2_C1SC[5]=0
					TPM_CnSC_MSA_MASK |         		///< MSA=TPM2_C1SC[4]=0
					TPM_CnSC_ELSB_MASK |        		///< ELSB=TPM2_C1SC[3]=0
					TPM_CnSC_ELSA_MASK );       		///< ELSA=TPM2_C1SC[2]=0
				TPM0_C1SC &= ~(TPM_CnSC_MSB_MASK |      ///< MSB=TPM0_C1SC[5]=0
					TPM_CnSC_MSA_MASK |         		///< MSA=TPM0_C1SC[4]=0
					TPM_CnSC_ELSB_MASK |        		///< ELSB=TPM0_C1SC[3]=0
					TPM_CnSC_ELSA_MASK );       		///< ELSA=TPM0_C1SC[2]=0
				
				delay10us(100000);
			}
		}
	}
	
}
